@@ -1,10 +1,14 @@
 # -*- coding: utf-8 -*-
 
+import asyncio
+
 from django.contrib.sessions.middleware import SessionMiddleware
 from django.core.handlers.asgi import ASGIRequest
 from django.middleware.csrf import CsrfViewMiddleware
+from django.utils.decorators import sync_and_async_middleware
 
 
+@sync_and_async_middleware
 class WebsocketOhjain:
   '''
   Ohjain, joka asettaa pyynnölle URI-määritteen `websocket` siten,
@@ -18,8 +22,15 @@ class WebsocketOhjain:
   '''
   def __init__(self, get_response):
     self.get_response = get_response
+    # Asetetaan tämä ohjain näyttämään ulospäin asynkroniselta silloin,
+    # kun signaalitien seuraava ohjain on sellainen.
+    if asyncio.iscoroutinefunction(self.get_response):
+      self._is_coroutine = asyncio.coroutines._is_coroutine
 
   def __call__(self, request):
+    # Huomaa, että funktiokutsu palauttaa joko synkronisen tuloksen
+    # tai asynkronisen alirutiinin sen mukaan, mitä signaalitien
+    # seuraava ohjain palauttaa.
     if isinstance(request, ASGIRequest):
       request.websocket = (
         f'{"wss" if request.is_secure() else "ws"}://{request.get_host()}'
@@ -39,22 +50,8 @@ class OhitaPaluusanoma:
 class CsrfOhjain(OhitaPaluusanoma, CsrfViewMiddleware):
   def process_view(self, request, callback, callback_args, callback_kwargs):
     '''
-    Lisätään Websocket-pyyntöön metodi `_tarkista_csrf` pyyntödatan
-    mukana luettavan CSRF-tunnisteen tarkistamiseen.
-
-    Ohitetaan tavanomainen CSRF-tarkistus POST-datasta (super).
+    Ohitetaan tavanomainen CSRF-tarkistus POST-datasta.
     '''
-    # pylint: disable=protected-access
-    from django.middleware.csrf import (
-      _compare_salted_tokens,
-      _sanitize_token,
-    )
-    def _tarkista_csrf(csrf_token):
-      return _compare_salted_tokens(
-        _sanitize_token(csrf_token), request.META.get('CSRF_COOKIE')
-      )
-    request._tarkista_csrf = _tarkista_csrf
-    # def process_view
   # class CsrfOhjain
 
 
@@ -72,7 +69,6 @@ WEBSOCKET_MIDDLEWARE = {
   'django.middleware.gzip.GZipMiddleware': False,
   'django.middleware.security.SecurityMiddleware' : False,
   'django.middleware.common.CommonMiddleware': False,
-  'django.contrib.messages.middleware.MessageMiddleware': False,
   'django.middleware.clickjacking.XFrameOptionsMiddleware': False,
   'django_hosts.middleware.HostsResponseMiddleware': False,
 
@@ -82,6 +78,7 @@ WEBSOCKET_MIDDLEWARE = {
   'django.contrib.sessions.middleware.SessionMiddleware': \
     'pistoke.ohjain.IstuntoOhjain',
   'django.contrib.auth.middleware.AuthenticationMiddleware': True,
+  'django.contrib.messages.middleware.MessageMiddleware': True,
   'impersonate.middleware.ImpersonateMiddleware': True,
   'silk.middleware.SilkyMiddleware': True,
   'pistoke.ohjain.WebsocketOhjain': True,
